using System;
using System.Drawing;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using System.Windows.Forms.Design;
using System.Data;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using System.Globalization;
using System.Threading;
using System.Resources;
using System.Diagnostics;
using System.Net;

namespace Microsoft.Exchange.Tools.IMFArchiveManager
{
	/// <summary>
	/// frmMain is the main class of IMFArchiveManager
	/// </summary>
	public class frmMain : System.Windows.Forms.Form
	{
		#region Class Globals
        delegate void SetWorkerList(ListViewItem[] lvi);
        delegate void SetUpdate(int count);
        private Thread updateThread = null;

		private System.Windows.Forms.TreeView treeView1;
		private System.Windows.Forms.ImageList imageList1;
		private System.ComponentModel.IContainer components;
		private System.Windows.Forms.Button btnRefresh;
		private System.Windows.Forms.Button btnDelete;
		private System.Windows.Forms.Button btnReSubmit;
		private System.Windows.Forms.RichTextBox lstMessage;
		private System.Windows.Forms.MainMenu mainMenu1;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuItem4;
		private System.Windows.Forms.MenuItem menuItem5;
		private System.Windows.Forms.MenuItem menuItem6;
		private System.Windows.Forms.MenuItem menuItem7;
		private System.Windows.Forms.MenuItem menuItem8;
		private System.Windows.Forms.MenuItem menuItem9;
		private System.Windows.Forms.MenuItem menuItem10;
		private System.Windows.Forms.MenuItem menuItem11;
		private System.Windows.Forms.MenuItem menuItem12;
		private System.Windows.Forms.MenuItem menuItem13;

		private static string m_xmlPath = "IMFAM.xml";
		public static string m_Pickup = "";
		public static string m_Archive = "";
		public static string m_From = "";
		public static string m_To = "";
		public static bool m_XStrip = false;
		public static bool m_XScl = false;
		public static bool m_DeleteOnReport = false;
		private static bool decodeException = false;
		private static string m_File = "";
		private static string m_Selected = "";
		private static int m_Left;
		private static int m_Top;
		private static int m_Width;
		private static int m_Height;
		private static int maxSearch;
		private static TreeNode m_Next_Node;
		private static DataSet ds;
		private static int lastColumn = 0;
		private static int lastIndex = 0;
		private static int currIndex = 0;
		private static SortOrder order = SortOrder.Ascending;
		public static MessageDict messages = new MessageDict();
        public static List<IPAddress> localIPs = new List<IPAddress>();
        private static Dictionary<IPAddress, string> resolvedIPs = new Dictionary<IPAddress, string>();
		private static string strPickupPath = "Pickup Directory: {0}";
		private static string strException = "Error: {0}\r\nStack: {1}\r\nSource: {2}";
		private static string strIOException = "IO Error Path: {0}, Error: {1}";
		private static string strCorruptXML = "{0} is corrupt.  Do you wish to recreate it?";
		private static string strCorruptXMLHeader = "Corrupt XML Detected";
		private static string strBadXML = "Archive and Pickup must be set to continue.";
		private static string strBadXMLHeader = "Path not set";
		private static string strArchive = "Select Archive Folder";
		private static string strPickup = "Select Pickup Folder";
		private static string strDeleted = "{0} Message(s) deleted";
		private static string strSubmit = "Message resubmitted";
		private static string strNoReport = "Report Settings must be set first!";
		private static string strNoReportHeader = "Missing Report Settings";
		private static string strReport = "Message report submitted";
		private static string strReportDelete = "Message reported and deleted";
		private static string strLastMessage = "";
		private static string strActiveUI;
        private static string strLoadedMessage;
		private static bool	bTreeView = false;
        private static bool bHideHdr = false;
		private int iTimer = 30000;
		private System.Windows.Forms.Timer timerRefresh;
		private System.Windows.Forms.Label lblDebug;
		private System.Windows.Forms.Button btnCopy;
		private System.Windows.Forms.Button btnReport;
		private System.Windows.Forms.MenuItem menuItem14;
		private System.Windows.Forms.MenuItem menuItem15;
		private System.Windows.Forms.FolderBrowserDialog fbdChooser;
		private System.Windows.Forms.Label lblStatus;
		private System.Windows.Forms.Label lblFrom;
		private System.Windows.Forms.TextBox txtFrom;
		private System.Windows.Forms.Label lblTo;
		private System.Windows.Forms.TextBox txtTo;
		private System.Windows.Forms.Label lblSubject;
		private System.Windows.Forms.TextBox txtSubject;
		private System.Windows.Forms.TextBox txtSCL;
		private System.Windows.Forms.Label lblSCL;
		private System.Windows.Forms.Label lblMessage;
		private System.Windows.Forms.TextBox txtDate;
		private System.Windows.Forms.Label lblDate;
		private System.Windows.Forms.Label lblTotalMessages;
		private System.Windows.Forms.Label lblMsgCount;
		private System.Windows.Forms.MenuItem menuItem16;
		private System.Windows.Forms.MenuItem menuItem17;
		private System.Windows.Forms.MenuItem menuItem18;
		private System.Windows.Forms.MenuItem menuItem19;
		private System.Windows.Forms.ListView listView1;
		public System.Windows.Forms.ColumnHeader colFrom;
		public System.Windows.Forms.ColumnHeader colSubject;
		public System.Windows.Forms.ColumnHeader colDate;
		public System.Windows.Forms.ColumnHeader colTo;
		public System.Windows.Forms.ColumnHeader colSCL;
		private System.Windows.Forms.ColumnHeader colFile;
		private System.Windows.Forms.Splitter splitter1;
		private System.Windows.Forms.Panel pBottom;
		private System.Windows.Forms.Panel pTop;
		private System.Windows.Forms.Panel pRight;
        private System.Windows.Forms.Panel pFill;
        private MenuItem menuItem20;
        private MenuItem menuItem21;
        public ColumnHeader colIP;
        private MenuItem menuItem22;
		private Report frmReport;
        private ColumnHeader colHost;
        private LocalIPs frmLocalIPs;

		#endregion

		public frmMain()
		{
			int x, y;
			//
			// Required for Windows Form Designer support
			//
			try
			{
				ResourceManager locRM = new ResourceManager(typeof(frmMain));
                strCorruptXML = Global.strCorruptXML;
                strCorruptXMLHeader = Global.strCorruptXMLHeader;
                strArchive = Global.strArchive;
                strPickup = Global.strPickup;
                strException = Global.strException;
                strBadXML = Global.strBadXML;
                strBadXMLHeader = Global.strBadXMLHeader;
				LoadXML();
				Thread.CurrentThread.CurrentUICulture = new CultureInfo(strActiveUI);
				frmReport = new Report(strActiveUI);
                frmLocalIPs = new LocalIPs();
				locRM = new ResourceManager(typeof(frmMain));
				strPickupPath = Global.strPickupPath;
				strException = Global.strException;
				strIOException = Global.strIOException;
				strArchive = Global.strArchive;
				strPickup = Global.strPickup;
				strDeleted = Global.strDeleted;
				strSubmit = Global.strSubmit;
				strNoReport = Global.strNoReport;
				strNoReportHeader = Global.strNoReportHeader;
				strReport = Global.strReport;
				strReportDelete = Global.strReportDelete;
				InitializeComponent();
				x = (int) ds.Tables[0].Rows[0]["Left"];
				y = (int) ds.Tables[0].Rows[0]["Top"];
				this.Location = new Point(x,y);
				x = (int) ds.Tables[0].Rows[0]["Width"];
				y = (int) ds.Tables[0].Rows[0]["Height"];
				this.Size = new Size(x,y);
				this.colFile.Width = (int) ds.Tables[3].Rows[0]["FileSize"];
				this.colFrom.Width = (int) ds.Tables[3].Rows[0]["FromSize"];
				this.colTo.Width = (int) ds.Tables[3].Rows[0]["ToSize"];
				this.colSubject.Width = (int) ds.Tables[3].Rows[0]["SubjectSize"];
				this.colDate.Width = (int) ds.Tables[3].Rows[0]["DateSize"];
				this.colSCL.Width = (int) ds.Tables[3].Rows[0]["SCLSize"];
				this.splitter1.SplitPosition = (int) ds.Tables[0].Rows[0]["SplitLeft"];
				this.WindowState = (FormWindowState) ds.Tables[0].Rows[0]["WindowState"];
                this.menuItem20.Checked = bHideHdr;
				if (bTreeView)
				{
					menuItem18.Checked = true;
					menuItem19.Checked = false;
					treeView1.Visible = true;
					listView1.Visible = false;
				}
				else
				{
					menuItem18.Checked = false;
					menuItem19.Checked = true;
					treeView1.Visible = false;
					listView1.Visible = true;
				}
				this.lblStatus.Text = "UI Culture: " + strActiveUI;
				timerRefresh.Interval = iTimer;
				this.Visible = true;
				this.Select();
				if (treeView1.Visible)
					InitializeTree();
				else
					InitializeList();
				frmReport.FormDone += new FormDoneHandler(this.FormDone);
                frmLocalIPs.FormDone += new FormDoneHandler(this.FormDone);
			}
			catch (Exception e)
			{
				System.IO.File.OpenWrite("error.log");
				MessageBox.Show(string.Format(strException, e.Message, e.StackTrace, e.Source),"Error Main()",MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
		}


		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(frmMain));
            this.treeView1 = new System.Windows.Forms.TreeView();
            this.imageList1 = new System.Windows.Forms.ImageList(this.components);
            this.btnRefresh = new System.Windows.Forms.Button();
            this.btnDelete = new System.Windows.Forms.Button();
            this.btnReSubmit = new System.Windows.Forms.Button();
            this.btnCopy = new System.Windows.Forms.Button();
            this.lstMessage = new System.Windows.Forms.RichTextBox();
            this.mainMenu1 = new System.Windows.Forms.MainMenu(this.components);
            this.menuItem1 = new System.Windows.Forms.MenuItem();
            this.menuItem3 = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.menuItem6 = new System.Windows.Forms.MenuItem();
            this.menuItem7 = new System.Windows.Forms.MenuItem();
            this.menuItem8 = new System.Windows.Forms.MenuItem();
            this.menuItem9 = new System.Windows.Forms.MenuItem();
            this.menuItem10 = new System.Windows.Forms.MenuItem();
            this.menuItem11 = new System.Windows.Forms.MenuItem();
            this.menuItem12 = new System.Windows.Forms.MenuItem();
            this.menuItem13 = new System.Windows.Forms.MenuItem();
            this.menuItem14 = new System.Windows.Forms.MenuItem();
            this.menuItem15 = new System.Windows.Forms.MenuItem();
            this.menuItem22 = new System.Windows.Forms.MenuItem();
            this.menuItem16 = new System.Windows.Forms.MenuItem();
            this.menuItem17 = new System.Windows.Forms.MenuItem();
            this.menuItem18 = new System.Windows.Forms.MenuItem();
            this.menuItem19 = new System.Windows.Forms.MenuItem();
            this.menuItem21 = new System.Windows.Forms.MenuItem();
            this.menuItem20 = new System.Windows.Forms.MenuItem();
            this.menuItem4 = new System.Windows.Forms.MenuItem();
            this.menuItem5 = new System.Windows.Forms.MenuItem();
            this.timerRefresh = new System.Windows.Forms.Timer(this.components);
            this.lblDebug = new System.Windows.Forms.Label();
            this.btnReport = new System.Windows.Forms.Button();
            this.fbdChooser = new System.Windows.Forms.FolderBrowserDialog();
            this.lblStatus = new System.Windows.Forms.Label();
            this.txtFrom = new System.Windows.Forms.TextBox();
            this.lblFrom = new System.Windows.Forms.Label();
            this.lblTo = new System.Windows.Forms.Label();
            this.txtTo = new System.Windows.Forms.TextBox();
            this.lblSubject = new System.Windows.Forms.Label();
            this.txtSubject = new System.Windows.Forms.TextBox();
            this.txtSCL = new System.Windows.Forms.TextBox();
            this.lblSCL = new System.Windows.Forms.Label();
            this.lblMessage = new System.Windows.Forms.Label();
            this.txtDate = new System.Windows.Forms.TextBox();
            this.lblDate = new System.Windows.Forms.Label();
            this.lblTotalMessages = new System.Windows.Forms.Label();
            this.lblMsgCount = new System.Windows.Forms.Label();
            this.listView1 = new System.Windows.Forms.ListView();
            this.colFile = new System.Windows.Forms.ColumnHeader();
            this.colFrom = new System.Windows.Forms.ColumnHeader();
            this.colTo = new System.Windows.Forms.ColumnHeader();
            this.colSubject = new System.Windows.Forms.ColumnHeader();
            this.colDate = new System.Windows.Forms.ColumnHeader();
            this.colSCL = new System.Windows.Forms.ColumnHeader();
            this.colIP = new System.Windows.Forms.ColumnHeader();
            this.pBottom = new System.Windows.Forms.Panel();
            this.pTop = new System.Windows.Forms.Panel();
            this.pRight = new System.Windows.Forms.Panel();
            this.pFill = new System.Windows.Forms.Panel();
            this.splitter1 = new System.Windows.Forms.Splitter();
            this.colHost = new System.Windows.Forms.ColumnHeader();
            this.pBottom.SuspendLayout();
            this.pTop.SuspendLayout();
            this.pRight.SuspendLayout();
            this.pFill.SuspendLayout();
            this.SuspendLayout();
            // 
            // treeView1
            // 
            resources.ApplyResources(this.treeView1, "treeView1");
            this.treeView1.ImageList = this.imageList1;
            this.treeView1.ItemHeight = 16;
            this.treeView1.Name = "treeView1";
            this.treeView1.DoubleClick += new System.EventHandler(this.listView1_DoubleClick);
            this.treeView1.AfterSelect += new System.Windows.Forms.TreeViewEventHandler(this.treeView1_AfterSelect);
            this.treeView1.KeyDown += new System.Windows.Forms.KeyEventHandler(this.treeView1_KeyDown);
            // 
            // imageList1
            // 
            this.imageList1.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("imageList1.ImageStream")));
            this.imageList1.TransparentColor = System.Drawing.Color.Transparent;
            this.imageList1.Images.SetKeyName(0, "");
            this.imageList1.Images.SetKeyName(1, "");
            this.imageList1.Images.SetKeyName(2, "");
            this.imageList1.Images.SetKeyName(3, "");
            // 
            // btnRefresh
            // 
            resources.ApplyResources(this.btnRefresh, "btnRefresh");
            this.btnRefresh.Name = "btnRefresh";
            this.btnRefresh.Click += new System.EventHandler(this.btnRefresh_Click);
            // 
            // btnDelete
            // 
            resources.ApplyResources(this.btnDelete, "btnDelete");
            this.btnDelete.DialogResult = System.Windows.Forms.DialogResult.Cancel;
            this.btnDelete.Name = "btnDelete";
            this.btnDelete.Click += new System.EventHandler(this.btnDelete_Click);
            // 
            // btnReSubmit
            // 
            resources.ApplyResources(this.btnReSubmit, "btnReSubmit");
            this.btnReSubmit.Name = "btnReSubmit";
            this.btnReSubmit.Click += new System.EventHandler(this.btnReSubmit_Click);
            // 
            // btnCopy
            // 
            resources.ApplyResources(this.btnCopy, "btnCopy");
            this.btnCopy.Name = "btnCopy";
            this.btnCopy.Click += new System.EventHandler(this.btnCopy_Click);
            // 
            // lstMessage
            // 
            resources.ApplyResources(this.lstMessage, "lstMessage");
            this.lstMessage.Name = "lstMessage";
            // 
            // mainMenu1
            // 
            this.mainMenu1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem1,
            this.menuItem2,
            this.menuItem11,
            this.menuItem4});
            // 
            // menuItem1
            // 
            this.menuItem1.Index = 0;
            this.menuItem1.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem3});
            resources.ApplyResources(this.menuItem1, "menuItem1");
            // 
            // menuItem3
            // 
            this.menuItem3.Index = 0;
            resources.ApplyResources(this.menuItem3, "menuItem3");
            this.menuItem3.Click += new System.EventHandler(this.menuItem3_Click);
            // 
            // menuItem2
            // 
            this.menuItem2.Index = 1;
            this.menuItem2.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem6,
            this.menuItem7,
            this.menuItem8,
            this.menuItem9,
            this.menuItem10});
            resources.ApplyResources(this.menuItem2, "menuItem2");
            // 
            // menuItem6
            // 
            this.menuItem6.Index = 0;
            resources.ApplyResources(this.menuItem6, "menuItem6");
            this.menuItem6.Click += new System.EventHandler(this.menuItem6_Click);
            // 
            // menuItem7
            // 
            this.menuItem7.Index = 1;
            resources.ApplyResources(this.menuItem7, "menuItem7");
            this.menuItem7.Click += new System.EventHandler(this.menuItem7_Click);
            // 
            // menuItem8
            // 
            this.menuItem8.Index = 2;
            resources.ApplyResources(this.menuItem8, "menuItem8");
            this.menuItem8.Click += new System.EventHandler(this.menuItem8_Click);
            // 
            // menuItem9
            // 
            this.menuItem9.Index = 3;
            resources.ApplyResources(this.menuItem9, "menuItem9");
            // 
            // menuItem10
            // 
            this.menuItem10.Index = 4;
            resources.ApplyResources(this.menuItem10, "menuItem10");
            this.menuItem10.Click += new System.EventHandler(this.menuItem10_Click);
            // 
            // menuItem11
            // 
            this.menuItem11.Index = 2;
            this.menuItem11.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem12,
            this.menuItem13,
            this.menuItem14,
            this.menuItem15,
            this.menuItem22,
            this.menuItem16,
            this.menuItem17,
            this.menuItem20});
            resources.ApplyResources(this.menuItem11, "menuItem11");
            // 
            // menuItem12
            // 
            this.menuItem12.Index = 0;
            resources.ApplyResources(this.menuItem12, "menuItem12");
            this.menuItem12.Click += new System.EventHandler(this.menuItem12_Click);
            // 
            // menuItem13
            // 
            this.menuItem13.Index = 1;
            resources.ApplyResources(this.menuItem13, "menuItem13");
            this.menuItem13.Click += new System.EventHandler(this.menuItem13_Click);
            // 
            // menuItem14
            // 
            this.menuItem14.Index = 2;
            resources.ApplyResources(this.menuItem14, "menuItem14");
            // 
            // menuItem15
            // 
            this.menuItem15.Index = 3;
            resources.ApplyResources(this.menuItem15, "menuItem15");
            this.menuItem15.Click += new System.EventHandler(this.menuItem15_Click);
            // 
            // menuItem22
            // 
            this.menuItem22.Index = 4;
            resources.ApplyResources(this.menuItem22, "menuItem22");
            this.menuItem22.Click += new System.EventHandler(this.menuItem22_Click);
            // 
            // menuItem16
            // 
            this.menuItem16.Index = 5;
            resources.ApplyResources(this.menuItem16, "menuItem16");
            // 
            // menuItem17
            // 
            this.menuItem17.Index = 6;
            this.menuItem17.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem18,
            this.menuItem19,
            this.menuItem21});
            resources.ApplyResources(this.menuItem17, "menuItem17");
            // 
            // menuItem18
            // 
            this.menuItem18.Checked = true;
            this.menuItem18.Index = 0;
            resources.ApplyResources(this.menuItem18, "menuItem18");
            this.menuItem18.Click += new System.EventHandler(this.menuItem18_Click);
            // 
            // menuItem19
            // 
            this.menuItem19.Index = 1;
            resources.ApplyResources(this.menuItem19, "menuItem19");
            this.menuItem19.Click += new System.EventHandler(this.menuItem19_Click);
            // 
            // menuItem21
            // 
            this.menuItem21.Index = 2;
            resources.ApplyResources(this.menuItem21, "menuItem21");
            // 
            // menuItem20
            // 
            this.menuItem20.Checked = true;
            this.menuItem20.Index = 7;
            resources.ApplyResources(this.menuItem20, "menuItem20");
            this.menuItem20.Click += new System.EventHandler(this.menuItem20_Click);
            // 
            // menuItem4
            // 
            this.menuItem4.Index = 3;
            this.menuItem4.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
            this.menuItem5});
            resources.ApplyResources(this.menuItem4, "menuItem4");
            // 
            // menuItem5
            // 
            this.menuItem5.Index = 0;
            resources.ApplyResources(this.menuItem5, "menuItem5");
            this.menuItem5.Click += new System.EventHandler(this.menuItem5_Click);
            // 
            // timerRefresh
            // 
            this.timerRefresh.Enabled = true;
            this.timerRefresh.Interval = 30000;
            this.timerRefresh.Tick += new System.EventHandler(this.timerRefresh_Tick);
            // 
            // lblDebug
            // 
            resources.ApplyResources(this.lblDebug, "lblDebug");
            this.lblDebug.Name = "lblDebug";
            // 
            // btnReport
            // 
            resources.ApplyResources(this.btnReport, "btnReport");
            this.btnReport.BackColor = System.Drawing.SystemColors.Control;
            this.btnReport.Name = "btnReport";
            this.btnReport.UseVisualStyleBackColor = false;
            this.btnReport.Click += new System.EventHandler(this.btnReport_Click);
            // 
            // fbdChooser
            // 
            this.fbdChooser.ShowNewFolderButton = false;
            // 
            // lblStatus
            // 
            resources.ApplyResources(this.lblStatus, "lblStatus");
            this.lblStatus.Name = "lblStatus";
            // 
            // txtFrom
            // 
            resources.ApplyResources(this.txtFrom, "txtFrom");
            this.txtFrom.Name = "txtFrom";
            this.txtFrom.ReadOnly = true;
            // 
            // lblFrom
            // 
            resources.ApplyResources(this.lblFrom, "lblFrom");
            this.lblFrom.Name = "lblFrom";
            // 
            // lblTo
            // 
            resources.ApplyResources(this.lblTo, "lblTo");
            this.lblTo.Name = "lblTo";
            // 
            // txtTo
            // 
            resources.ApplyResources(this.txtTo, "txtTo");
            this.txtTo.Name = "txtTo";
            this.txtTo.ReadOnly = true;
            // 
            // lblSubject
            // 
            resources.ApplyResources(this.lblSubject, "lblSubject");
            this.lblSubject.Name = "lblSubject";
            // 
            // txtSubject
            // 
            resources.ApplyResources(this.txtSubject, "txtSubject");
            this.txtSubject.Name = "txtSubject";
            this.txtSubject.ReadOnly = true;
            // 
            // txtSCL
            // 
            resources.ApplyResources(this.txtSCL, "txtSCL");
            this.txtSCL.Name = "txtSCL";
            this.txtSCL.ReadOnly = true;
            // 
            // lblSCL
            // 
            resources.ApplyResources(this.lblSCL, "lblSCL");
            this.lblSCL.Name = "lblSCL";
            // 
            // lblMessage
            // 
            resources.ApplyResources(this.lblMessage, "lblMessage");
            this.lblMessage.Name = "lblMessage";
            // 
            // txtDate
            // 
            resources.ApplyResources(this.txtDate, "txtDate");
            this.txtDate.Name = "txtDate";
            this.txtDate.ReadOnly = true;
            // 
            // lblDate
            // 
            resources.ApplyResources(this.lblDate, "lblDate");
            this.lblDate.Name = "lblDate";
            // 
            // lblTotalMessages
            // 
            resources.ApplyResources(this.lblTotalMessages, "lblTotalMessages");
            this.lblTotalMessages.Name = "lblTotalMessages";
            // 
            // lblMsgCount
            // 
            resources.ApplyResources(this.lblMsgCount, "lblMsgCount");
            this.lblMsgCount.Name = "lblMsgCount";
            // 
            // listView1
            // 
            this.listView1.AllowColumnReorder = true;
            resources.ApplyResources(this.listView1, "listView1");
            this.listView1.Columns.AddRange(new System.Windows.Forms.ColumnHeader[] {
            this.colFile,
            this.colFrom,
            this.colTo,
            this.colSubject,
            this.colDate,
            this.colSCL,
            this.colIP,
            this.colHost});
            this.listView1.FullRowSelect = true;
            this.listView1.Name = "listView1";
            this.listView1.UseCompatibleStateImageBehavior = false;
            this.listView1.View = System.Windows.Forms.View.Details;
            this.listView1.DoubleClick += new System.EventHandler(this.listView1_DoubleClick);
            this.listView1.KeyDown += new System.Windows.Forms.KeyEventHandler(this.listView1_KeyDown_1);
            this.listView1.ColumnClick += new System.Windows.Forms.ColumnClickEventHandler(this.listView1_ColumnClick);
            this.listView1.KeyUp += new System.Windows.Forms.KeyEventHandler(this.listView1_KeyUp);
            this.listView1.Click += new System.EventHandler(this.listView1_Click);
            // 
            // colFile
            // 
            resources.ApplyResources(this.colFile, "colFile");
            // 
            // colFrom
            // 
            resources.ApplyResources(this.colFrom, "colFrom");
            // 
            // colTo
            // 
            resources.ApplyResources(this.colTo, "colTo");
            // 
            // colSubject
            // 
            resources.ApplyResources(this.colSubject, "colSubject");
            // 
            // colDate
            // 
            resources.ApplyResources(this.colDate, "colDate");
            // 
            // colSCL
            // 
            resources.ApplyResources(this.colSCL, "colSCL");
            // 
            // colIP
            // 
            resources.ApplyResources(this.colIP, "colIP");
            // 
            // pBottom
            // 
            this.pBottom.BackColor = System.Drawing.SystemColors.Control;
            this.pBottom.Controls.Add(this.lblDebug);
            this.pBottom.Controls.Add(this.lblStatus);
            this.pBottom.Controls.Add(this.btnReport);
            this.pBottom.Controls.Add(this.btnCopy);
            this.pBottom.Controls.Add(this.btnReSubmit);
            this.pBottom.Controls.Add(this.btnDelete);
            this.pBottom.Controls.Add(this.btnRefresh);
            resources.ApplyResources(this.pBottom, "pBottom");
            this.pBottom.Name = "pBottom";
            // 
            // pTop
            // 
            this.pTop.BackColor = System.Drawing.SystemColors.Control;
            this.pTop.Controls.Add(this.lblMsgCount);
            this.pTop.Controls.Add(this.lblTotalMessages);
            resources.ApplyResources(this.pTop, "pTop");
            this.pTop.Name = "pTop";
            // 
            // pRight
            // 
            this.pRight.BackColor = System.Drawing.SystemColors.Control;
            this.pRight.Controls.Add(this.listView1);
            this.pRight.Controls.Add(this.treeView1);
            resources.ApplyResources(this.pRight, "pRight");
            this.pRight.Name = "pRight";
            // 
            // pFill
            // 
            this.pFill.BackColor = System.Drawing.SystemColors.Control;
            this.pFill.Controls.Add(this.txtDate);
            this.pFill.Controls.Add(this.lblDate);
            this.pFill.Controls.Add(this.lblMessage);
            this.pFill.Controls.Add(this.txtSCL);
            this.pFill.Controls.Add(this.lblSCL);
            this.pFill.Controls.Add(this.txtSubject);
            this.pFill.Controls.Add(this.lblSubject);
            this.pFill.Controls.Add(this.txtTo);
            this.pFill.Controls.Add(this.lblTo);
            this.pFill.Controls.Add(this.lblFrom);
            this.pFill.Controls.Add(this.txtFrom);
            this.pFill.Controls.Add(this.lstMessage);
            resources.ApplyResources(this.pFill, "pFill");
            this.pFill.Name = "pFill";
            // 
            // splitter1
            // 
            this.splitter1.BackColor = System.Drawing.SystemColors.ButtonShadow;
            resources.ApplyResources(this.splitter1, "splitter1");
            this.splitter1.Name = "splitter1";
            this.splitter1.TabStop = false;
            // 
            // colHost
            // 
            resources.ApplyResources(this.colHost, "colHost");
            // 
            // frmMain
            // 
            resources.ApplyResources(this, "$this");
            this.Controls.Add(this.pFill);
            this.Controls.Add(this.splitter1);
            this.Controls.Add(this.pRight);
            this.Controls.Add(this.pTop);
            this.Controls.Add(this.pBottom);
            this.Menu = this.mainMenu1;
            this.Name = "frmMain";
            this.FormClosed += new System.Windows.Forms.FormClosedEventHandler(this.frmMain_FormClosed);
            this.Resize += new System.EventHandler(this.frmMain_Resize);
            this.Enter += new System.EventHandler(this.btnReport_Click);
            this.Move += new System.EventHandler(this.frmMain_Move);
            this.Closing += new System.ComponentModel.CancelEventHandler(this.frmMain_Closing);
            this.pBottom.ResumeLayout(false);
            this.pTop.ResumeLayout(false);
            this.pTop.PerformLayout();
            this.pRight.ResumeLayout(false);
            this.pFill.ResumeLayout(false);
            this.pFill.PerformLayout();
            this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new frmMain());
		}

		#region List Functions
		private void InitializeList()
		{
			listView1_Update();
			lblDebug.Text = string.Format(strPickupPath, m_Pickup);
		}

		private void listView1_Update()
		{
            if (updateThread == null || !updateThread.IsAlive)
            {
                updateThread = new Thread(new ThreadStart(UpdateTable));
                this.lblMsgCount.Text = this.listView1.Items.Count.ToString();
                updateThread.Start();
            }
		}

        private void AddLVI(ListViewItem[] lvis)
        {
            if (this.listView1.InvokeRequired)
            {
                SetWorkerList d = new SetWorkerList(AddLVI);
                this.Invoke(d, new object[] { lvis });
            }
            else
            {
                this.listView1.Items.AddRange(lvis);
                this.lblMsgCount.Text = this.listView1.Items.Count.ToString();
                Application.DoEvents();
            }
        }

        private void UpdateCount(int count)
        {
            if (this.lblMsgCount.InvokeRequired)
            {
                SetUpdate d = new SetUpdate(UpdateCount);
                this.Invoke(d, new object[] { count });
            }
            else
            {
                this.lblMsgCount.Text = ((Int32)(this.listView1.Items.Count + count)).ToString();
            }
        }

		private void UpdateTable()
		{
			string m_strFilter = "*.eml";

			Log("UpdateTable started");

			// Use the Directory class to get the files
			DirectoryInfo Dir = new DirectoryInfo(m_Archive);

            try
            {
                List<ListViewItem> lvis = new List<ListViewItem>();
                // Get each file name and add it as a file tree node
                FileInfo[] dfArr = Dir.GetFiles(m_strFilter);
                foreach (FileInfo drf in dfArr)
                {
                    if (!messages.Contains(drf.Name))
                    {
                        try
                        {
                            if (drf.Exists)
                            {
                                ListViewItem lvi = new ListViewItem(drf.Name);
                                //Crack file and get properties
                                MessageProps message = CrackMessage(drf);
                                lvi.SubItems.Add(message.From);
                                lvi.SubItems.Add(message.To);
                                lvi.SubItems.Add(message.Subject);
                                lvi.SubItems.Add(message.Date);
                                lvi.SubItems.Add(message.SCL);
                                lvi.SubItems.Add(message.IP.ToString());
                                if (resolvedIPs.ContainsKey(message.IP))
                                    lvi.SubItems.Add(resolvedIPs[message.IP]);
                                else
                                {
                                    string host = Dns.GetHostEntry(message.IP).HostName;
                                    resolvedIPs.Add(message.IP, host);
                                    lvi.SubItems.Add(host);
                                }
                                lvis.Add(lvi);
                                messages.Add(drf.Name, message);

                                // Update UI
                                UpdateCount(lvis.Count);
                                if (lvis.Count % 100 == 0)
                                {
                                    AddLVI(lvis.ToArray());
                                    lvis.Clear();
                                }
                            }
                        }
                        catch (FileNotFoundException eio)
                        {
                            // Just log this exception, don't pop-up as background thread could try to
                            // read a file that was just deleted on another thread.
                            Log(string.Format(strIOException, m_Archive, eio.ToString()));
                        }
                    }
                    Thread.Sleep(0);
                }
                if (lvis.Count > 0)
                    AddLVI(lvis.ToArray());
            }
            catch (ThreadAbortException)
            {
                // Just need to exit
            }
            catch (IOException eio)
            {
                Log(string.Format(strIOException, m_Archive, eio.ToString()));
                MessageBox.Show(string.Format(strIOException, m_Archive, eio.Message));
            }
            catch (Exception en)
            {
                Log(string.Format("Error Populate node: {0}", en.ToString()));
                MessageBox.Show(string.Format(strException, en.Message, en.StackTrace, en.Source), "Error PopulateNode()", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            finally
            {
                Log("UpdateTable finished");
            }
		}

		private void LoadMessage()
		{
			// Populate text field with file

			if (!strLastMessage.Equals(m_File))
			{
				// New Message, so clear the Message window
				resetForm();
				this.lblStatus.Text = "UI Culture: " + strActiveUI;
				strLastMessage = m_File;
				if (System.IO.File.Exists(m_File))
				{
					this.lstMessage.Visible = false;
					this.lstMessage.LoadFile(m_File,RichTextBoxStreamType.PlainText);

					// figure how far to search for headers, Max 10,000 characters
					if (this.lstMessage.Text.Length < 10000)
						maxSearch = this.lstMessage.Text.Length;
					else
						maxSearch = 10000;
					int iEOH = this.lstMessage.Text.IndexOf("\n\n", 0, maxSearch) + 1;
					if (iEOH > 0)
						maxSearch = iEOH;

					// Search for X-SCL and move it to EOH
					int iStart = this.lstMessage.Text.ToUpper().IndexOf("X-SCL:", 0, maxSearch);
					if (iStart > 0)
					{
						int iEnd = this.lstMessage.Text.IndexOf("\n", iStart, maxSearch - iStart) + 1;
						if (iEnd > 0)
						{
							string strSCL = this.lstMessage.Text.Substring(iStart, iEnd - iStart);
							this.txtSCL.Text = strSCL.Substring(7, strSCL.Length - 8);
							if (iEOH > 0)
							{
								this.lstMessage.Text = this.lstMessage.Text.Remove(iStart,iEnd - iStart);
								// Check if Null in header truncated
								if (this.lstMessage.Text.Length < iEOH)
								{
									this.lblStatus.Text = String.Format("Message has embeded NULL at char {0}.  Headers can not be automatically trimmed.", this.lstMessage.Text.Length - 1);
									this.lstMessage.LoadFile(m_File,RichTextBoxStreamType.PlainText);
								}
								else
								{
									// Need to adjust iEOH since we removed a line
									iEOH = iEOH - (iEnd - iStart);
									// Search for End of Headers
                                    if (iEOH > 0)
                                    {
                                        this.lstMessage.Text = this.lstMessage.Text.Insert(iEOH, strSCL);
                                        iEOH += strSCL.Length;
                                    }
								}
							}
						}
					}
                    // Save off formatted message
                    strLoadedMessage = this.lstMessage.Text;

                    // Check if we want to strip headers
                    if (iEOH > 0)
                    {
                        if (!menuItem20.Checked)
                            this.lstMessage.Text = strLoadedMessage.Substring(iEOH + 1);
                    }

					// Search for the rest of the headers
					this.txtFrom.Text = messages[listView1.Items[currIndex].Text].From;
					this.txtTo.Text = messages[listView1.Items[currIndex].Text].To;
					this.txtSubject.Text = messages[listView1.Items[currIndex].Text].Subject;
					this.txtDate.Text = messages[listView1.Items[currIndex].Text].Date;

					this.lstMessage.Visible = true;

					this.btnDelete.Enabled = true;
					this.btnReport.Enabled = true;
					this.btnReSubmit.Enabled = true;
					this.btnCopy.Enabled = true;
					lastIndex = currIndex;
				}
				else
				{
					messages.Remove(listView1.Items[currIndex].Text);
					listView1.Items[currIndex].Remove();
					listView1_Update();
				}
			}
		}

		private void listView1_Click(object sender, System.EventArgs e)
		{
			if( this.listView1.FocusedItem != null)
			{
				m_File = m_Archive + "\\" + this.listView1.FocusedItem.Text;
				currIndex = this.listView1.FocusedItem.Index;
				LoadMessage();
			}
		}

		private void listView1_KeyUp(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			if( this.listView1.FocusedItem != null)
			{
				m_File = m_Archive + "\\" + this.listView1.FocusedItem.Text;
				currIndex = this.listView1.FocusedItem.Index;
				LoadMessage();
			}
		}

		public static MessageProps CrackMessage(FileInfo file)
		{
			StringWriter sw = new StringWriter();
			MessageProps props = new MessageProps();

			//Set Exception detection to false
			decodeException = false;

			// figure how far to search for headers, Max 10,000 characters
			if (file.Length < 10000)
				maxSearch = (int) file.Length;
			else
				maxSearch = 10000;
            using (StreamReader sr = file.OpenText())
            {
                char[] chars = new char[maxSearch];
                sr.Read(chars, 0, maxSearch);
                sw.Write(chars);
                sr.Close();
            }
			string strHeaders = sw.ToString();

			int iEOH = strHeaders.IndexOf("\r\n\r\n", 0, maxSearch);
			// if we found the body separator, then set maxSearch to index + 2 (to include /r/n)
			if (iEOH > -1)
				maxSearch = iEOH + 2;

			// Search for the the headers
			try
			{
				props.SCL = FindField("x-scl:", strHeaders).TrimEnd((new char[] {'\r'}));
				props.From = DecodeField(FindField("from:", strHeaders)).TrimEnd((new char[] {'\r'}));
				props.To = DecodeField(FindField("to:", strHeaders)).TrimEnd((new char[] {'\r'}));
				props.Subject = DecodeField(FindField("subject:", strHeaders)).TrimEnd((new char[] {'\r'}));
				props.Date = FixDate(FindField("\t", strHeaders).TrimEnd((new char[] {'\r'})));
                props.IP = FindIP(strHeaders);
				if (decodeException)
					Log(String.Format("Exception in File: {0}", file.FullName));
			}
			catch (Exception e)
			{
				Log(String.Format("Exception: {0}\r\n\r\nFile: {1}", e.ToString(), file.FullName));
				throw;
			}

			return props;
		}

		private void listView1_ColumnClick(object sender, System.Windows.Forms.ColumnClickEventArgs e)
		{
			if(lastColumn != e.Column || order != SortOrder.Ascending)
			{
				lastColumn = e.Column;
				order = SortOrder.Ascending;
			}
			else
			{
				order = SortOrder.Descending;
			}
			if (e.Column == 4)
				this.listView1.ListViewItemSorter = new ListViewDateComparer(e.Column, order);
			else
				this.listView1.ListViewItemSorter = new ListViewItemComparer(e.Column, order);

			// Call the sort method to manually sort the column based on the ListViewItemComparer implementation.
			listView1.Sort();		
		}

		private void Reselect()
		{
			if (this.listView1.Items.Count > 0)
			{
				if (this.listView1.Items.Count <= lastIndex)
					lastIndex = this.listView1.Items.Count - 1;
				foreach (ListViewItem lvi in this.listView1.SelectedItems)
				{
					lvi.Selected = false;
				}
				this.listView1.Items[lastIndex].Selected = true;
				this.listView1.Items[lastIndex].Focused = true;
				this.listView1.Focus();
				m_File = m_Archive + "\\" + this.listView1.FocusedItem.Text;
				currIndex = this.listView1.FocusedItem.Index;
				LoadMessage();
			}
			else
				resetForm();
		}

		#endregion

		#region Tree Functions
		private void InitializeTree()
		{
			try
			{
				if (m_Archive.Length == 0)
				{
					string [] strDrives = Directory.GetLogicalDrives();
					foreach (string strDrive in strDrives)
					{
						TreeNode RootNode = new TreeNode(strDrive, 3, 3);
						treeView1.Nodes.Add(RootNode);
						ClearTree(RootNode);
					}
				}
				else
				{
					treeView1_Update();
				}
				lblDebug.Text = string.Format(strPickupPath, m_Pickup);
			}
			catch(Exception ex)
			{
				MessageBox.Show(string.Format(strException, ex.Message, ex.StackTrace, ex.Source),"Error InitializeTree()",MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
		}

		public void ClearTree(TreeNode RootNode)
		{
			timerRefresh.Stop();
			RootNode.Nodes.Clear();
			PopulateNode(RootNode);
			timerRefresh.Start();
		}

		private void treeView1_AfterSelect(object sender, System.Windows.Forms.TreeViewEventArgs e)
		{
			this.btnCopy.Enabled = false;
			m_File = "";

			// Check to see if the selected node is already populated
			if (e.Node.Nodes.Count > 0 || e.Node.ImageIndex == 3)
			{
				// No Message, so clear the Message window
				resetForm();
				return;
			}

			if (e.Node.ImageIndex == 2)
			{
				// Set the next Node
				if (e.Node.NextNode != null)
					m_Next_Node = e.Node.NextNode;
				else
					m_Next_Node = e.Node.PrevNode;

				// Populate text field with file
				m_File = BuildDirectory(e.Node);
				m_File = m_File.Remove(m_File.Length - 1, 1);

				if (!strLastMessage.Equals(m_File))
				{
					// New Message, so clear the Message window
					resetForm();
					strLastMessage = m_File;
					if (System.IO.File.Exists(m_File))
					{
						this.lstMessage.Visible = false;
						this.lstMessage.LoadFile(m_File,RichTextBoxStreamType.PlainText);

						// figure how far to search for headers, Max 10,000 characters
						if (this.lstMessage.Text.Length < 10000)
							maxSearch = this.lstMessage.Text.Length;
						else
							maxSearch = 10000;
						int iEOH = this.lstMessage.Text.IndexOf("\n\n", 0, maxSearch) + 1;
						if (iEOH > 0)
							maxSearch = iEOH;

						// Search for X-SCL and move it to EOH
						int iStart = this.lstMessage.Text.ToUpper().IndexOf("X-SCL:", 0, maxSearch);
						if (iStart > 0)
						{
							int iEnd = this.lstMessage.Text.IndexOf("\n", iStart, maxSearch - iStart) + 1;
							if (iEnd > 0)
							{
								string strSCL = this.lstMessage.Text.Substring(iStart, iEnd - iStart);
								this.txtSCL.Text = strSCL.Substring(7, strSCL.Length - 8);
								if (iEOH > 0)
								{
									this.lstMessage.Text = this.lstMessage.Text.Remove(iStart,iEnd - iStart);
									// Need to adjust iEOH since we removed a line
									iEOH = iEOH - (iEnd - iStart);
									// Search for End of Headers
                                    if (iEOH > 0)
                                    {
                                        this.lstMessage.Text = this.lstMessage.Text.Insert(iEOH, strSCL);
                                        iEOH += strSCL.Length;
                                    }
								}
							}
						}

                        // Save off formatted message
                        strLoadedMessage = this.lstMessage.Text;

                        // Check if we want to strip headers
                        if (iEOH > 0)
                        {
                            if (!menuItem20.Checked)
                                this.lstMessage.Text = strLoadedMessage.Substring(iEOH + 1);
                        }

                        // Search for the rest of the headers
						this.txtFrom.Text = DecodeField(FindField("from:", this.lstMessage.Text.Substring(0, maxSearch)));
						this.txtTo.Text = DecodeField(FindField("to:", this.lstMessage.Text.Substring(0, maxSearch)));
						this.txtSubject.Text = DecodeField(FindField("subject:",  this.lstMessage.Text.Substring(0, maxSearch)));
						this.txtDate.Text = FixDate(FindField("\t",  this.lstMessage.Text.Substring(0, maxSearch)));

						this.lstMessage.Visible = true;

						this.btnDelete.Enabled = true;
						this.btnReport.Enabled = true;
						this.btnReSubmit.Enabled = true;
						this.btnCopy.Enabled = true;
					}
					else
					{
						treeView1_Update();
					}
				}
			}
			else
			{
				// The node is not populated, so populate the selected node
				PopulateNode(e.Node);
			}
		}

        public static IPAddress FindIP(string message)
        {
            string result = "";
            string field = "received:";
            IPAddress ip;
            int pos = 0;
            string lowMsg = message.ToLower();
            string strIP = "";
            int iStart = lowMsg.IndexOf(field, pos, maxSearch);
            int iEnd = 0;
            int iIPEnd = 0;
            while (iStart > -1)
            {
                // Check first to see if the match is at the beginning or is preceded by newline
                if (iStart == 0 || message.Substring(iStart - 1, 1).Equals("\n"))
                {
                    // remove field name
                    iStart = iStart + field.Length;
                    iEnd = message.IndexOf("\n", iStart, maxSearch - iStart);
                    if (iEnd > 0)
                    {
                        // Found Receive header
                        result = message.Substring(iStart, iEnd - iStart).Trim();
                        iStart = result.IndexOf("[");
                        iIPEnd = result.IndexOf("]");
                        // check for IP string
                        if (iIPEnd > iStart && iStart > -1)
                        {
                            // Check for valid IP address
                            strIP = result.Substring(iStart + 1, iIPEnd - iStart - 1);
                            if (IPAddress.TryParse(strIP,out ip) && !localIPs.Contains(ip))
                            {
                                return ip;
                            }
                        }
                        // Look for next header
                        iStart = lowMsg.IndexOf(field, iEnd, maxSearch - iEnd);
                    }
                    else
                    {
                        // Header doesn't end with \n so we don't have the whole line
                        iStart = -1;
                    }
                }
                else
                {
                    // We found header match inside a line, so need to ignore and search again
                    pos = iStart + field.Length;
                    iStart = lowMsg.IndexOf(field, pos, maxSearch - pos);
                }
            }
            return IPAddress.None;
        }

		public static string FindField(string field, string message)
		{
			string result = "";
			string lowMsg = message.ToLower();
			int pos = 0;
			int iStart = lowMsg.IndexOf(field, pos, maxSearch);
			while (iStart > -1)
			{
				// Check first to see if the match is at the beginning or is preceded by newline
				if (iStart == 0 || message.Substring(iStart - 1, 1).Equals("\n"))
				{
					// remove field name
					iStart = iStart + field.Length;
					int iEnd = message.IndexOf("\n", iStart, maxSearch - iStart);
					if (iEnd > 0)
						result = message.Substring(iStart, iEnd - iStart).Trim();
					iStart = -1;
				}
				else
				{
					pos = iStart + field.Length;
					iStart = lowMsg.IndexOf(field, pos, maxSearch - pos);
				}
			}
			return result;
		}

		public static string DecodeField(string field)
		{
			string result = field;
			int isoStart = field.IndexOf("=?");
			int isoEnd = field.IndexOf("?=") - isoStart -2;
			if (isoStart > -1 && isoEnd > -1)
			{
				string text = field.Substring(isoStart + 2, isoEnd);
				string[] parts = field.Substring(isoStart + 2, isoEnd).Split('?');

				// Only continue parsing if there are 3 parts
				// charset?encoding?text
				if (parts.Length == 3)
				{
					try
					{
						Encoding coder;
						if (parts[0].ToLower().StartsWith("cp"))
							coder = Encoding.GetEncoding(int.Parse(parts[0].Substring(2)));
						else
							coder = Encoding.GetEncoding(parts[0]);
						byte[] bSubject;
						if (parts[1].ToLower().Equals("b"))
							bSubject = Convert.FromBase64String(parts[2]);
						else
							bSubject = QPGetString(parts[2]);
						result = coder.GetString(bSubject);
						if (isoStart + isoEnd + 4 < field.Length)
							result += field.Substring(isoStart + isoEnd + 4);
						if (isoStart > 0)
							result = field.Substring(0, isoStart) + result;
					}
					catch (ArgumentException)
					{
						decodeException = true;
						//this.lblStatus.Text = "Encoding.GetEncoding(" + parts[0] +") failed, invalid code page";
                        Log(string.Format("{0} - String: {1}", "Encoding.GetEncoding(" + parts[0] + ") failed, invalid code page", field));
					}
					catch (FormatException)
					{
						decodeException = true;
						//this.lblStatus.Text = "Convert.FromBase64String(" + parts[2] +") failed, invalid length";
                        Log(string.Format("{0} - String: {1}", "Convert.FromBase64String(" + parts[2] + ") failed, invalid length", field));
					}
				}
			}
			return result;
		}

		public static byte[] QPGetString(string line)
		{
			if (line == null)
				throw new ArgumentNullException();

			ArrayList byteArray = new ArrayList();

			// look for =XX where xx is hex
			Regex reg = new Regex("(\\=([0-9A-F][0-9A-F]))", RegexOptions.IgnoreCase);
			for (int i = 0; i < line.Length; i++)
			{
				if(line.Length >= i + 3 &&
					Char.Equals(line[i],'=') &&
					reg.IsMatch(line.Substring(i,3)))
				{
					byteArray.Add(Convert.ToByte(int.Parse(line.Substring(i+1,2), NumberStyles.HexNumber)));
					// skip two chars
					i += 2;
				}
				else
				{
					byteArray.Add(Convert.ToByte(char.Parse(line.Substring(i,1))));
				}
			}
			return (byte[]) byteArray.ToArray(typeof(byte));
		}

		public static string FixDate(string date)
		{
			string result = date;
			try
			{
				IFormatProvider format = CultureInfo.CreateSpecificCulture("en-us");
				string[] formats = {"ddd, d MMM yyyy HH:mm:ss zz00", "ddd, dd MMM yyyy HH:mm:ss zz00"};
				DateTime dtDate = DateTime.ParseExact(date,formats,format,DateTimeStyles.AdjustToUniversal);
				result = dtDate.ToLocalTime().ToString("g", Thread.CurrentThread.CurrentUICulture);
			}
			catch
			{
				result = date;
			}
			return result;
		}

		protected string BuildDirectory(TreeNode aNode)
		{
			TreeNode theNode = aNode;
			string strDir = "";

			// Cycle through the node and all its parents to build the full path of the directory

			while (theNode != null)
			{
				if (theNode.Text[theNode.Text.Length - 1] != '\\')
				{
					strDir = "\\" + strDir;
				}
				strDir = theNode.Text + strDir;
				theNode = theNode.Parent;
			}
			return strDir;
		}

		protected void PopulateNode(TreeNode aNode)
		{
			string m_strFilter = "*.eml";

			this.Cursor = Cursors.WaitCursor;

			// Build the directory Tree string from the tree node
			string strDir = BuildDirectory(aNode);

			// Use the Directory class to get the files and directory names in the directory
			DirectoryInfo Dir = new DirectoryInfo(strDir);

			// Get each directory name and add it as a directory folder tree node
			try
			{

				// print out the names of the directories

				DirectoryInfo[] diArr = Dir.GetDirectories();
				foreach (DirectoryInfo dri in diArr)
				{
					TreeNode ChildNode = new TreeNode(dri.Name, 1,0);
					aNode.Nodes.Add(ChildNode);
					if (m_Selected.CompareTo(dri.Name) == 0)
						this.treeView1.SelectedNode = ChildNode;
					Application.DoEvents();
				}

				// Get each file name and add it as a file tree node
				FileInfo[] dfArr = Dir.GetFiles(m_strFilter);
				lblMsgCount.Text = dfArr.Length.ToString();
				foreach (FileInfo drf in dfArr)
				{
					TreeNode ChildNode = new TreeNode(drf.Name, 2,2);
					aNode.Nodes.Add(ChildNode);
					if (m_Selected.CompareTo(drf.DirectoryName + "\\" + drf.Name) == 0)
						this.treeView1.SelectedNode = ChildNode;
					Application.DoEvents();
				}
				// Set the next Node
				if (this.treeView1.SelectedNode != null)
				{
					if (this.treeView1.SelectedNode.NextNode != null)
						m_Next_Node = this.treeView1.SelectedNode.NextNode;
					else
						m_Next_Node = this.treeView1.SelectedNode.PrevNode;
				}
			}
			catch (IOException eio)
			{
				Log(string.Format(strIOException, strDir, eio.Message));
				MessageBox.Show(string.Format(strIOException, strDir, eio.Message));
			}
			catch (Exception en)
			{
				Log(string.Format("Error Populate node: {0}",en.ToString()));
				MessageBox.Show(string.Format(strException, en.Message, en.StackTrace, en.Source),"Error PopulateNode()",MessageBoxButtons.OK,MessageBoxIcon.Error);
			}

			// Show the children nodes
			aNode.Expand();

			// restore the cursor
			this.Cursor = Cursors.Arrow;
		}

		private void treeView1_Update()
		{
			if (m_Archive.Length > 0)
			{
				if (this.treeView1.SelectedNode != null)
					m_Selected = this.treeView1.SelectedNode.FullPath;
				else
					resetForm();
				treeView1.Nodes.Clear();
				TreeNode RootNode = new TreeNode(m_Archive, 0, 1);
				treeView1.Nodes.Add(RootNode);
				ClearTree(RootNode);
				treeView1.Focus();
			}
		}

		#endregion

		#region XML Loading and Saving
		private void LoadXML()
		{
			int x, y;
			FormWindowState state;

			ds = new DataSet("Settings");

			try
			{
				ds.ReadXml(m_xmlPath);
				x = (int) ds.Tables[0].Rows[0]["Left"];
				y = (int) ds.Tables[0].Rows[0]["Top"];
				x = (int) ds.Tables[0].Rows[0]["Width"];
				y = (int) ds.Tables[0].Rows[0]["Height"];
				state = (FormWindowState) ds.Tables[0].Rows[0]["WindowState"];
				m_Archive = ds.Tables[1].Rows[0]["IMFArchive"].ToString();
				m_Pickup = ds.Tables[1].Rows[0]["Pickup"].ToString();
				m_From = ds.Tables[2].Rows[0]["ReportFrom"].ToString();
				m_To = ds.Tables[2].Rows[0]["ReportTo"].ToString();
				m_XStrip = (bool) ds.Tables[2].Rows[0]["ReportStripXHeaders"];
				m_XScl = (bool) ds.Tables[2].Rows[0]["ReportStripSCL"];
				m_DeleteOnReport = (bool) ds.Tables[2].Rows[0]["DeleteOnReport"];
				bTreeView = (bool) ds.Tables[3].Rows[0]["TreeView"];
				iTimer = (int) ds.Tables[0].Rows[0]["Refresh"];
				strActiveUI = (string) ds.Tables[0].Rows[0]["Language"];
				if (strActiveUI.Length == 0)
					strActiveUI = Thread.CurrentThread.CurrentUICulture.ToString();
				x = (int) ds.Tables[0].Rows[0]["SplitLeft"];
				x = (int) ds.Tables[3].Rows[0]["FileSize"];
				x = (int) ds.Tables[3].Rows[0]["FromSize"];
				x = (int) ds.Tables[3].Rows[0]["ToSize"];
				x = (int) ds.Tables[3].Rows[0]["SubjectSize"];
				x = (int) ds.Tables[3].Rows[0]["DateSize"];
				x = (int) ds.Tables[3].Rows[0]["SCLSize"];
                bHideHdr = (bool)ds.Tables[3].Rows[0]["HideHeaders"];
			}
			catch
			{
				this.Select();
				// If XML exists, popup message indicating its corrupt
				if (File.Exists(m_xmlPath))
				{
					DialogResult dr = MessageBox.Show(string.Format(strCorruptXML, m_xmlPath),strCorruptXMLHeader,MessageBoxButtons.YesNo, MessageBoxIcon.Error);
					if (dr == DialogResult.No)
					{
						System.Environment.Exit(0);
						return;
					}
				}

				// Create XML
				
				// Prompt for Archive and Pickup
				if(m_Archive.Length == 0)
				{
					try
					{
						fbdChooser = new FolderBrowserDialog();
						fbdChooser.Description = strArchive;
						fbdChooser.SelectedPath = m_Archive;
						if (fbdChooser.ShowDialog() == DialogResult.OK)
							m_Archive = fbdChooser.SelectedPath;
					}
					catch (Exception ex)
					{
						MessageBox.Show(string.Format(strException, ex.Message, ex.StackTrace, ex.Source),"Error SelectArchive()",MessageBoxButtons.OK,MessageBoxIcon.Error);
					}
				}
				if(m_Pickup.Length == 0)
				{
					try
					{
						fbdChooser = new FolderBrowserDialog();
						fbdChooser.Description = strPickup;
						fbdChooser.SelectedPath = m_Pickup;
						if (fbdChooser.ShowDialog() == DialogResult.OK)
							m_Pickup = fbdChooser.SelectedPath;
					}
					catch (Exception ex)
					{
						MessageBox.Show(string.Format(strException, ex.Message, ex.StackTrace, ex.Source),"Error SelectPickup()",MessageBoxButtons.OK,MessageBoxIcon.Error);
					}
				}
				if (m_Archive.Length == 0 || m_Pickup.Length == 0)
				{
					MessageBox.Show(strBadXML,strBadXMLHeader,MessageBoxButtons.OK,MessageBoxIcon.Stop);
					System.Environment.Exit(0);
				}

				// Form settings
				ds.Tables.Clear();
				ds.Clear();
				ds.Tables.Add("Form");
				ds.Tables[0].Columns.Add("Left", typeof(int));
				ds.Tables[0].Columns.Add("Top", typeof(int));
				ds.Tables[0].Columns.Add("Width", typeof(int));
				ds.Tables[0].Columns.Add("Height", typeof(int));
				ds.Tables[0].Columns.Add("SplitLeft", typeof(int));
				ds.Tables[0].Columns.Add("WindowState", typeof(int));
				ds.Tables[0].Columns.Add("Refresh", typeof(int));
				ds.Tables[0].Columns.Add("Language", typeof(string));

				// Path settings
				ds.Tables.Add("Paths");
				ds.Tables[1].Columns.Add("IMFArchive", typeof(string));
				ds.Tables[1].Columns.Add("Pickup", typeof(string));

				// Report settings
				ds.Tables.Add("Report_Settings");
				ds.Tables[2].Columns.Add("ReportFrom", typeof(string));
				ds.Tables[2].Columns.Add("ReportTo", typeof(string));
				ds.Tables[2].Columns.Add("ReportStripXHeaders", typeof(bool));
				ds.Tables[2].Columns.Add("ReportStripSCL", typeof(bool));
				ds.Tables[2].Columns.Add("DeleteOnReport", typeof(bool));

				// View Settings
				ds.Tables.Add("View_Settings");
				ds.Tables[3].Columns.Add("TreeView", typeof(bool));
				ds.Tables[3].Columns.Add("FileSize", typeof(int));
				ds.Tables[3].Columns.Add("FromSize", typeof(int));
				ds.Tables[3].Columns.Add("ToSize", typeof(int));
				ds.Tables[3].Columns.Add("SubjectSize", typeof(int));
				ds.Tables[3].Columns.Add("DateSize", typeof(int));
				ds.Tables[3].Columns.Add("SCLSize", typeof(int));
                ds.Tables[3].Columns.Add("HideHeaders", typeof(bool));

				// Add Default Form settings
				DataRow workRow = ds.Tables[0].NewRow();
				workRow["Left"] = 0;
				workRow["Top"] = 0;
				workRow["Width"] = 656;
				workRow["Height"] = 533;
				workRow["SplitLeft"] = 224;
				workRow["WindowState"] = (int) FormWindowState.Normal;
				workRow["Refresh"] = 30000;
				workRow["Language"] = "";
				ds.Tables[0].Rows.Add(workRow);

				// Add Default Path settings
				workRow = ds.Tables[1].NewRow();
				workRow["IMFArchive"] = m_Archive;
				workRow["Pickup"] = m_Pickup;
				ds.Tables[1].Rows.Add(workRow);

				// Add Default Report settings
				workRow = ds.Tables[2].NewRow();
				workRow["ReportFrom"] = "";
				workRow["ReportTo"] = "";
				workRow["ReportStripXHeaders"] = false;
				workRow["ReportStripSCL"] = false;
				workRow["DeleteOnReport"] = false;
				ds.Tables[2].Rows.Add(workRow);

				// Add Default View Settings
				workRow = ds.Tables[3].NewRow();
				workRow["TreeView"] = false;
				workRow["FileSize"] = 0;
				workRow["FromSize"] = 60;
				workRow["ToSize"] = 60;
				workRow["SubjectSize"] = 60;
				workRow["DateSize"] = 60;
				workRow["SCLSize"] = 60;
                workRow["HideHeaders"] = false;
				ds.Tables[3].Rows.Add(workRow);

				// Write default info and schema
				ds.WriteXml(m_xmlPath, XmlWriteMode.WriteSchema);

				strActiveUI = Thread.CurrentThread.CurrentUICulture.ToString();
			}
		}

		private void SaveXML()
		{
			ds.Tables[0].Rows[0]["Left"] = m_Left;
			ds.Tables[0].Rows[0]["Top"] = m_Top;
			ds.Tables[0].Rows[0]["Width"] = m_Width;
			ds.Tables[0].Rows[0]["Height"] = m_Height;
			ds.Tables[0].Rows[0]["SplitLeft"] = this.splitter1.SplitPosition;
			ds.Tables[0].Rows[0]["WindowState"] = this.WindowState;
			ds.Tables[0].Rows[0]["Refresh"] = timerRefresh.Interval;
			ds.Tables[1].Rows[0]["IMFArchive"] = m_Archive;
			ds.Tables[1].Rows[0]["Pickup"] = m_Pickup;
			ds.Tables[2].Rows[0]["ReportFrom"] = m_From;
			ds.Tables[2].Rows[0]["ReportTo"] = m_To;
			ds.Tables[2].Rows[0]["ReportStripXHeaders"] = m_XStrip;
			ds.Tables[2].Rows[0]["ReportStripSCL"] = m_XScl;
			ds.Tables[2].Rows[0]["DeleteOnReport"] = m_DeleteOnReport;
			ds.Tables[3].Rows[0]["TreeView"] = menuItem18.Checked;
			ds.Tables[3].Rows[0]["FileSize"] = this.colFile.Width;
			ds.Tables[3].Rows[0]["FromSize"] = this.colFrom.Width;
			ds.Tables[3].Rows[0]["ToSize"] = this.colTo.Width;
			ds.Tables[3].Rows[0]["SubjectSize"] = this.colSubject.Width;
			ds.Tables[3].Rows[0]["DateSize"] = this.colDate.Width;
			ds.Tables[3].Rows[0]["SCLSize"] = this.colSCL.Width;
            ds.Tables[3].Rows[0]["HideHeaders"] = menuItem20.Checked;
			ds.WriteXml(m_xmlPath, XmlWriteMode.WriteSchema);
		}

		#endregion

		#region Form Main
		private void frmMain_Resize(object sender, System.EventArgs e)
		{
			if (this.WindowState == FormWindowState.Normal)
			{
				m_Left = this.Left;
				m_Top = this.Top;
				m_Width = this.Width;
				m_Height = this.Height;
			}
		}

		private void frmMain_Move(object sender, System.EventArgs e)
		{
			if (this.WindowState == FormWindowState.Normal)
			{
				m_Left = this.Left;
				m_Top = this.Top;
				m_Width = this.Width;
				m_Height = this.Height;
			}
		}
		private void frmMain_Closing(object sender, System.ComponentModel.CancelEventArgs e)
		{
			SaveXML();
		}

		private void FormDone(object sender, System.EventArgs e)
		{
			this.Enabled = true;
			this.Focus();
		}

        public static void Log
            (String logMessage)
		{
			StreamWriter w = File.AppendText("IMFAM.log");
			w.Write("\r\nLog Entry : ");
			w.WriteLine("{0} {1}", DateTime.Now.ToLongTimeString(),
				DateTime.Now.ToLongDateString());
			w.WriteLine("  :");
			w.WriteLine("  :{0}", logMessage);
			w.WriteLine ("-------------------------------");
			// Update the underlying file.
			w.Flush();
			w.Close();
		}

		#endregion

		#region Menu Actions

		private void menuItem3_Click(object sender, System.EventArgs e)
		{
			this.Close();
			Application.Exit();
		}


		private void menuItem6_Click(object sender, System.EventArgs e)
		{
			this.lstMessage.Focus();
			this.lstMessage.Cut();
		}

		private void menuItem7_Click(object sender, System.EventArgs e)
		{
			this.lstMessage.Focus();
			this.lstMessage.Copy();
		}

		private void menuItem8_Click(object sender, System.EventArgs e)
		{
			this.lstMessage.Focus();
			this.lstMessage.Paste();
		}

		private void menuItem10_Click(object sender, System.EventArgs e)
		{
			this.lstMessage.Focus();
			this.lstMessage.SelectAll();
		}

		private void menuItem12_Click(object sender, System.EventArgs e)
		{
			try
			{
				fbdChooser.Description = strArchive;
				fbdChooser.SelectedPath = m_Archive;
				if (fbdChooser.ShowDialog() == DialogResult.OK)
				{
					m_Archive = fbdChooser.SelectedPath;
					btnRefresh_Click(sender, e);
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(string.Format(strException, ex.Message, ex.StackTrace, ex.Source),"Error SelectArchive()",MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
		}

		private void menuItem13_Click(object sender, System.EventArgs e)
		{
			try
			{
				fbdChooser.Description = strPickup;
				fbdChooser.SelectedPath = m_Pickup;
				if (fbdChooser.ShowDialog() == DialogResult.OK)
				{
					m_Pickup = fbdChooser.SelectedPath;
					lblDebug.Text = string.Format(strPickupPath, m_Pickup);
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(string.Format(strException, ex.Message, ex.StackTrace, ex.Source),"Error SelectPickup()",MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
		}

		private void menuItem15_Click(object sender, System.EventArgs e)
		{
		
			this.Enabled = false;

			frmReport.TopMost = true;
			frmReport.Enabled = true;
			frmReport.Visible = true;
			frmReport.Show();
		}
		private void menuItem18_Click(object sender, System.EventArgs e)
		{
			menuItem18.Checked = true;
			menuItem19.Checked = false;
			treeView1.Visible = true;
			listView1.Visible = false;
			treeView1_Update();
		}

		private void menuItem19_Click(object sender, System.EventArgs e)
		{
			menuItem18.Checked = false;
			menuItem19.Checked = true;
			treeView1.Visible = false;
			listView1.Visible = true;
			listView1.Items.Clear();
			messages.Clear();
			listView1_Update();
			Reselect();
		}

        private void menuItem22_Click(object sender, EventArgs e)
        {
            this.Enabled = false;

            frmLocalIPs.TopMost = true;
            frmLocalIPs.Enabled = true;
            frmLocalIPs.Visible = true;
            frmLocalIPs.Show();
        }
        
        private void menuItem5_Click(object sender, System.EventArgs e)
		{
			System.Diagnostics.Process.Start("http://workspaces.gotdotnet.com/imfarchive");
		}
		#endregion

		#region Button Actions

		private void btnCopy_Click(object sender, System.EventArgs e)
		{
			this.lstMessage.Focus();
			this.lstMessage.SelectAll();
			this.lstMessage.Copy();
		}

		private void btnRefresh_Click(object sender, System.EventArgs e)
		{
			resetForm();
			if (this.treeView1.Visible)
				treeView1_Update();
			else
			{
				listView1.Items.Clear();
				messages.Clear();
				listView1_Update();
				Reselect();
			}
		}


		private void timerRefresh_Tick(object sender, System.EventArgs e)
		{
			if (this.treeView1.Visible)
				treeView1_Update();
			else
				listView1_Update();
		}

		private int btnBegin()
		{
			this.Cursor = Cursors.WaitCursor;
			this.lstMessage.Hide();
			this.listView1.BeginUpdate();
			return this.listView1.SelectedItems[0].Index;
		}

		private void btnComplete(int index)
		{
			lastIndex = index;
			this.Cursor = Cursors.Arrow;
			this.listView1.EndUpdate();
			this.treeView1.Visible = false;
			this.lstMessage.Show();
			listView1_Update();
			Reselect();
		}

		private void btnDelete_Click(object sender, System.EventArgs e)
		{
			int iCount = 0;
			try
			{
				resetForm();
				if (this.treeView1.Visible)
				{
					iCount++;
					try
					{
						System.IO.File.Delete(m_File);
					}
					catch (IOException)
					{
						if(m_File.Length >0)
							MessageBox.Show("Bad File: " + m_File,"Bad File", MessageBoxButtons.OK);
					}
					this.treeView1.SelectedNode = m_Next_Node;
					treeView1_Update();
				}
				else
				{
					int index = btnBegin();
					foreach (ListViewItem lsi in this.listView1.SelectedItems)
					{
						iCount++;
						try
						{
							System.IO.File.Delete(m_Archive + "\\" + lsi.Text);
						}
						catch (IOException)
						{
							if(lsi.Text.Length > 0)
								MessageBox.Show("Bad File: " + m_Archive + "\\" + lsi.Text,"Bad File", MessageBoxButtons.OK);
						}
						messages.Remove(lsi.Text);
						lsi.Remove();
						this.lblStatus.Text = string.Format(strDeleted,iCount);
						Application.DoEvents();
					}
					btnComplete(index);
				}
				this.lblStatus.Text = string.Format(strDeleted,iCount);
			}
			catch (Exception ex)
			{
				MessageBox.Show(string.Format(strException, ex.Message, ex.StackTrace, ex.Source),"Error Delete",MessageBoxButtons.OK,MessageBoxIcon.Error);
			}
		}

		private void btnReSubmit_Click(object sender, System.EventArgs e)
		{
			if (this.treeView1.Visible)
			{
				System.IO.File.Move(m_File, m_Pickup + m_File.Substring(m_File.LastIndexOf('\\')));
				this.treeView1.SelectedNode = m_Next_Node;
				treeView1_Update();
			}
			else
			{
				if (this.listView1.SelectedItems.Count > 0)
				{
					int index = btnBegin();
					foreach (ListViewItem lvi in this.listView1.SelectedItems)
					{
						m_File = m_Archive + @"\" + lvi.Text;
						currIndex = lvi.Index;
						System.IO.File.Move(m_File, m_Pickup + @"\" + lvi.Text);
						messages.Remove(lvi.Text);
						lvi.Remove();
					}
					btnComplete(index);
				}
			}
			this.lblStatus.Text = strSubmit;
		}

		private void resetForm()
		{
			strLastMessage = "";
			this.lstMessage.Clear();
			this.btnDelete.Enabled = false;
			this.btnReport.Enabled = false;
			this.btnReSubmit.Enabled = false;
			this.txtSCL.Text = "";
			this.txtFrom.Text = "";
			this.txtTo.Text = "";
			this.txtSubject.Text = "";
			this.txtDate.Text = "";
		}

		private void btnReport_Click(object sender, System.EventArgs e)
		{
			int iCount = 0;
			if (m_From.Length == 0 || m_To.Length == 0)
			{
				MessageBox.Show(strNoReport, strNoReportHeader, MessageBoxButtons.OK,MessageBoxIcon.Exclamation);
			}
			else
			{
				lblStatus.Text = strReport;
				Application.DoEvents();
				if (this.treeView1.Visible)
				{
					iCount++;
					string strFile = m_File.Substring(m_File.LastIndexOf('\\') + 1);
					GenerateReport(strFile);
					if (m_DeleteOnReport)
					{
						File.Delete(m_File);
						resetForm();
						this.treeView1.SelectedNode = m_Next_Node;
						treeView1_Update();
					}
				}
				else
				{
					if(this.listView1.SelectedItems.Count > 0)
					{
						int index = btnBegin();
						foreach (ListViewItem lvi in this.listView1.SelectedItems)
						{
							iCount++;
							m_File = m_Archive + "\\" + lvi.Text;
							currIndex = lvi.Index;
							LoadMessage();
							GenerateReport(lvi.Text);
							if (m_DeleteOnReport)
							{
								File.Delete(m_File);
								resetForm();
								messages.Remove(lvi.Text);
								this.listView1.Items[lvi.Index].Remove();
							}
						}
						btnComplete(index);
					}
				}
				this.lblStatus.Text = string.Format(strReportDelete, iCount);
			}
		}

		private void GenerateReport(string strFile)
		{
			string strBound = "--=_NextPart_" + System.Guid.NewGuid().ToString();
			FileStream fs = new FileStream(m_Pickup + "\\" + strFile, FileMode.Create);
			StreamWriter sw = new StreamWriter(fs);
			// write x-headers for pickup
			sw.WriteLine("x-Sender: " + m_From);
			sw.WriteLine("x-Receiver: " + m_To);
			// write address headers
			sw.WriteLine("From: <" + m_From + ">");
			sw.WriteLine("To: <" + m_To + ">");
			sw.WriteLine("Subject: Spam report");
			// write mime headers
			sw.WriteLine("MIME-Version: 1.0");
			sw.WriteLine("Content-Type: multipart/mixed;");
			sw.WriteLine("\tboundary=\"" + strBound + "\"");
			sw.WriteLine();
			// write body
			sw.WriteLine("This is a multi-part message in MIME format.");
			sw.WriteLine();
			// write mime headers
			sw.WriteLine("--" + strBound);
			sw.WriteLine("Content-Type: text/plain;");
			sw.WriteLine("\tcharset=\"iso-8859-1\"");
			sw.WriteLine("Content-Transfer-Encoding: 8bit");
			sw.WriteLine();
			sw.WriteLine();
			// write attachment mime
			sw.WriteLine("--" + strBound);
			sw.WriteLine("Content-Type: message/rfc822;");
			sw.WriteLine("\tname=\"" + strFile + "\"");
			sw.WriteLine("Content-Transfer-Encoding: 8bit");
			sw.WriteLine("Content-Disposition: attachment;");
			sw.WriteLine("\tfilename=\"" + strFile + "\"");
			sw.WriteLine();
			sw.Flush();
			// strip x headers if requested
			if (m_XStrip)
			{
                int iStart = strLoadedMessage.ToLower().IndexOf("received:");
				if (iStart > 0)
                    strLoadedMessage = strLoadedMessage.Substring(iStart);
			}
			// strip X-SCL if requested
			if (m_XScl)
			{
                int iHead = strLoadedMessage.ToLower().IndexOf("x-scl: ") - 1;
				if (iHead > 0)
				{
                    int iBody = strLoadedMessage.IndexOf("\n", iHead + 1);
					if (iBody > 0)
					{
                        string temp = strLoadedMessage.Substring(0, iHead);
                        this.lstMessage.Text = temp + strLoadedMessage.Substring(iBody);
					}
				}
			}
			// write the message
			this.lstMessage.SaveFile(fs,RichTextBoxStreamType.PlainText);
			sw.WriteLine();
			// close the mime
			sw.WriteLine("--" + strBound + "--");
			sw.WriteLine();
			sw.Close();
		}

		private void treeView1_KeyDown(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			if(e.KeyData == Keys.Delete && btnDelete.Enabled == true)
				btnDelete_Click(sender, e);
		}

		private void listView1_KeyDown_1(object sender, System.Windows.Forms.KeyEventArgs e)
		{
			if(e.KeyData == Keys.Delete && btnDelete.Enabled == true)
				btnDelete_Click(sender, e);
			if (e.KeyData == Keys.Enter && this.btnReport.Enabled == true)
				btnReport_Click(sender, e);
		}

        private void menuItem20_Click(object sender, EventArgs e)
        {
            if (menuItem20.Checked)
                menuItem20.Checked = false;
            else
                menuItem20.Checked = true;

        }

        private void listView1_DoubleClick(object sender, EventArgs e)
        {
            if (m_File.Length > 0)
                Process.Start(m_File);
        }
        #endregion

        private void frmMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            updateThread.Abort();
        }



	}

	# region Helper Classes
	public class MessageProps
	{
		private string from = "";
		private string to = "";
		private string subject = "";
		private string date = "";
		private string scl = "";
        private IPAddress ip;
        private string sid = "";

		public MessageProps()
		{
			this.from = "";
			this.to = "";
			this.subject = "";
			this.date = "";
			this.scl = "";
            this.ip = null;
		}
		public MessageProps(string From, string To, string Subject, string Date, string SCL)
		{
			this.from = From;
			this.to = To;
			this.subject = Subject;
			this.date = Date;
			this.scl = SCL;
		}

		public string From
		{
			get {return this.from;}
			set {this.from = value;}
		}
		public string To
		{
			get {return this.to;}
			set {this.to = value;}
		}
		public string Subject
		{
			get {return this.subject;}
			set {this.subject = value;}
		}
		public string Date
		{
			get {return this.date;}
			set {this.date = value;}
		}
		public string SCL
		{
			get {return this.scl;}
			set {this.scl = value;}
		}
        public IPAddress IP
        {
            get {return this.ip;}
            set {this.ip = value;}
        }
        public string SID
        {
            get {return this.sid;}
            set {this.sid = value;}
        }
	}


	public class MessageDict : DictionaryBase
	{
		public MessageProps this[string key]
		{
			get
			{
				if (this.Dictionary.Contains(key))
				{
					return (MessageProps) this.Dictionary[key];
				}
				else
					return null;
			}
			set
			{
				if (this.Dictionary.Contains(key))
					this.Dictionary[key] = value;
				else
					this.Dictionary.Add(key, value);
			}
		}
		public void Add(string key, MessageProps props)
		{
			this.Dictionary.Add(key, props);
		}
		public void Remove(string key)
		{
			if (this.Dictionary.Contains(key))
				this.Dictionary.Remove(key);
		}
		public bool Contains(string key)
		{
			return this.Dictionary.Contains(key);
		}
		public ICollection Keys
		{
			get {return this.Dictionary.Keys;}
		}
	}

	// Implements the manual sorting of items by columns.
	class ListViewItemComparer : IComparer 
	{
		private int col;
		private SortOrder order;
		public ListViewItemComparer() 
		{
			col=0;
			order = SortOrder.Ascending;
		}
		public ListViewItemComparer(int column, SortOrder order) 
		{
			col=column;
			this.order = order;
		}
		public int Compare(object x, object y) 
		{
			int returnVal = String.Compare(((ListViewItem)x).SubItems[col].Text, ((ListViewItem)y).SubItems[col].Text);
			// If Descending, then turn swap sign
			if(order == SortOrder.Descending)
				returnVal *= -1;
			return returnVal;
		}
	}

	// Implements the manual sorting of dates by columns.
	class ListViewDateComparer : IComparer 
	{
		private int col;
		private SortOrder order;
		public ListViewDateComparer() 
		{
			col=0;
			order = SortOrder.Ascending;
		}
		public ListViewDateComparer(int column, SortOrder order) 
		{
			col=column;
			this.order = order;
		}
		public int Compare(object x, object y) 
		{
			int returnVal;
			// Determine whether the type being compared is a date type.
			try
			{
				// Parse the two objects passed as a parameter as a DateTime.
				string format = "g";
				IFormatProvider provider = System.Globalization.CultureInfo.CurrentCulture;
				System.DateTime firstDate = 
					DateTime.ParseExact(((ListViewItem)x).SubItems[col].Text, format, provider);
				System.DateTime secondDate = 
					DateTime.ParseExact(((ListViewItem)y).SubItems[col].Text, format, provider);
				// Compare the two dates.
				returnVal = DateTime.Compare(firstDate, secondDate);
			}
				// If neither compared object has a valid date format, compare
				// as a string.
			catch 
			{
				// Compare the two items as a string.
				returnVal = String.Compare(((ListViewItem)x).SubItems[col].Text,
					((ListViewItem)y).SubItems[col].Text);
			}
			// Determine whether the sort order is descending.
			if (order == SortOrder.Descending)
				// Invert the value returned by String.Compare.
				returnVal *= -1;
			return returnVal;
		}
	}
	#endregion
}
